package cn.com.qmth.stmms.api.interceptor;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import cn.com.qmth.stmms.api.utils.AuthInfoUtils;
import cn.com.qmth.stmms.biz.api.auth.annotation.AuthValidate;
import cn.com.qmth.stmms.biz.api.auth.interfaces.AuthValidator;
import cn.com.qmth.stmms.biz.api.auth.model.AuthInfo;
import cn.com.qmth.stmms.biz.api.auth.service.AuthInfoService;
import cn.com.qmth.stmms.biz.api.auth.service.validator.AuthValidateService;
import cn.com.qmth.stmms.common.utils.RequestUtils;

/**
 * 远程服务接口子模块操作拦截器
 * 
 * @author LS
 * 
 */
public class ApiInterceptor extends HandlerInterceptorAdapter {

    @Autowired
    private AuthInfoService infoService;

    @Autowired
    private AuthValidateService validateService;

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler,
            ModelAndView modelAndView) throws Exception {
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
            throws Exception {
        HandlerMethod method = (HandlerMethod) handler;
        AuthValidate validateConfig = method.getMethodAnnotation(AuthValidate.class);
        if (validateConfig != null) {
            AuthInfo auth = infoService.getAuthInfo(AuthInfoUtils.getAuthInfoValue(request));
            List<AuthValidator> validators = validateService.getValidators(validateConfig.value());
            if (validators != null) {
                boolean flag = true;
                HttpStatus status = null;
                String reason = null;
                for (AuthValidator validator : validators) {
                    try {
                        // 执行验证
                        flag = validator.validate(auth);
                    } catch (RuntimeException e) {
                        // 处理程序异常
                        flag = false;
                        ResponseStatus rs = e.getClass().getAnnotation(ResponseStatus.class);
                        if (rs != null) {
                            status = rs.value();
                            reason = rs.reason();
                        }
                    }
                    if (validateConfig.nand() && flag == false) {
                        // 与的情况，一个验证失败即退出
                        if (status == null) {
                            status = HttpStatus.UNAUTHORIZED;
                        }
                        break;
                    } else if (validateConfig.nand() == false && flag == true) {
                        // 或的情况，一个验证成功即退出
                        break;
                    }
                }
                if (!flag) {
                    // 验证失败返回错误码
                    if (status == null) {
                        status = HttpStatus.UNAUTHORIZED;
                    }
                    response.sendError(status.value(), StringUtils.trimToEmpty(reason));
                }
                RequestUtils.setApiUser(request, auth.getLoginUser());
                return flag;
            }
        }
        return true;
    }
}
